/*! \file positronimp.cpp
    \brief Die Implementation der Klasse \ref PositronImp

    Diese Datei wurde EINMAL automatisch durch Aufruf von
 
    uic -o positronimp.cpp -subimpl PositronImp positronimp.h positron.ui 
    
    erzeugt.
    
    Anschlieend wurden die Ereignissbehandlungsroutinen mit dem bentigten Code
    gefllt.
*/
#include "positronimp.h"
#include <math.h>
#include <qthread.h>
#ifdef _WS_WIN_
#include <windows.h>
#endif
#include "nrutil.h"
#include "globals.h"
#ifdef _WS_X11_
  #include <unistd.h>
#endif
#include <stdlib.h>
#include "helpwindow.h"
#include "kovarianzformimpl.h"
#include <qwindowsstyle.h>
#include <qstylesheet.h>
#include <qcursor.h>
#include "funcs.h"
#include "gnmcode.h"
#include "spektrumsimulierenimpl.h"
/* 
 *  Constructs a PositronImp which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */

PositronImp::PositronImp( QWidget* parent,  const char* name, WFlags fl )
    : Positron( parent, name, fl )
{
  init(); 
  initFonts(); 
}

PositronImp::~PositronImp()
{
    
}
/* 
 * public slot
 */
int PositronImp::initFonts()
{
  /* int PointSize=16;
  QString FontFamily="Utopia";
  QFile datei;
  QString msg="Datei daten/font kann nicht geffnet werden";
        datei.setName("daten/font");
        if (!datei.open(IO_ReadOnly)) { 
	  QMessageBox::information(0,"Fehler beim ffnen der Datei daten/font",msg,QMessageBox::Ok);
          return 0; 
        } else { 
          QTextStream instream(&datei);
          instream>>PointSize;
          //instream>>FontFamily;
          datei.close();
          return 1;
        }
        QFont BeendenButton_font(FontFamily);
	BeendenButton_font.setPointSize(PointSize);
	BeendenButton->setFont( BeendenButton_font ); 
	QFont Titel_font(  Titel->font() );
	Titel_font.setBold( TRUE );
	Titel->setFont( Titel_font ); 
	QFont HilfeButton_font(  HilfeButton->font() );
	HilfeButton_font.setPointSize(PointSize);
        HilfeButton_font.setFamily(FontFamily);
	HilfeButton->setFont( HilfeButton_font ); 
	QFont Notebook_font(  Notebook->font() );
	Notebook_font.setPointSize(PointSize-1);
        Notebook_font.setFamily(FontFamily);
	Notebook->setFont( Notebook_font ); 
	QFont Auswahl_BeschreibungLabel_font(  Auswahl_BeschreibungLabel->font() );
        Auswahl_BeschreibungLabel_font.setFamily(FontFamily);
	Auswahl_BeschreibungLabel->setFont( Auswahl_BeschreibungLabel_font ); 
	QFont Auswahl_SpektrumTitelLabel_font(  Auswahl_SpektrumTitelLabel->font() );
	Auswahl_SpektrumTitelLabel_font.setPointSize(PointSize);
        Auswahl_SpektrumTitelLabel_font.setFamily(FontFamily);
	Auswahl_SpektrumTitelLabel_font.setBold( TRUE );
	Auswahl_SpektrumTitelLabel->setFont( Auswahl_SpektrumTitelLabel_font ); 
	QFont Auswahl_TitelLabel_font(  Auswahl_TitelLabel->font() );
	Auswahl_TitelLabel_font.setFamily(FontFamily);
	Auswahl_TitelLabel_font.setBold( TRUE );
	Auswahl_TitelLabel->setFont( Auswahl_TitelLabel_font ); 
	QFont Startwerte_TitelLabel_font(  Startwerte_TitelLabel->font() );
	Startwerte_TitelLabel_font.setBold( TRUE );
        Startwerte_TitelLabel_font.setFamily(FontFamily);
	Startwerte_TitelLabel->setFont( Startwerte_TitelLabel_font ); 
	QFont TextLabel8_font(  TextLabel8->font() );
        TextLabel8_font.setFamily(FontFamily);
	TextLabel8->setFont( TextLabel8_font ); 
	QFont TextLabel7_font(  TextLabel7->font() );
        TextLabel7_font.setFamily(FontFamily);
	TextLabel7_font.setBold( TRUE );
	TextLabel7->setFont( TextLabel7_font ); 
	QFont TextLabel6_font(  TextLabel6->font() );
        TextLabel6_font.setFamily(FontFamily);
	TextLabel6->setFont( TextLabel6_font ); 
	QFont TextLabel5_font(  TextLabel5->font() );
	TextLabel5_font.setBold( TRUE );
        TextLabel5_font.setFamily(FontFamily);
	TextLabel5->setFont( TextLabel5_font ); 
	QFont TextLabel4_font(  TextLabel4->font() );
        TextLabel4_font.setFamily(FontFamily);
	TextLabel4->setFont( TextLabel4_font ); 
	QFont TextLabel3_font(  TextLabel3->font() );
	TextLabel3_font.setBold( TRUE );
        TextLabel3_font.setFamily(FontFamily);
	TextLabel3->setFont( TextLabel3_font ); 
	QFont TextLabel2_font(  TextLabel2->font() );
        TextLabel2_font.setFamily(FontFamily);
	TextLabel2->setFont( TextLabel2_font ); 
	QFont TextLabel1_font(  TextLabel1->font() );
	TextLabel1_font.setBold( TRUE );
        TextLabel1_font.setFamily(FontFamily);
	TextLabel1->setFont( TextLabel1_font ); 
	QFont Startwerte_langeBeschreibung_font(  Startwerte_langeBeschreibung->font() );
        Startwerte_langeBeschreibung_font.setFamily(FontFamily);
	Startwerte_langeBeschreibung->setFont( Startwerte_langeBeschreibung_font ); 
	QFont Startwerte_BeschreibungLabel_font(  Startwerte_BeschreibungLabel->font() );
        Startwerte_BeschreibungLabel_font.setFamily(FontFamily);
	Startwerte_BeschreibungLabel->setFont( Startwerte_BeschreibungLabel_font ); 
	QFont Startwerte_Label1_font(  Startwerte_Label1->font() );
        Startwerte_Label1_font.setFamily(FontFamily);
	Startwerte_Label1->setFont( Startwerte_Label1_font ); 
	QFont Verlauf_TitelLabel_font(  Verlauf_TitelLabel->font() );
	Verlauf_TitelLabel_font.setBold( TRUE );
        Verlauf_TitelLabel_font.setFamily(FontFamily);
	Verlauf_TitelLabel->setFont( Verlauf_TitelLabel_font ); 
	QFont Verlauf_BeschreibungLabel_font(  Verlauf_BeschreibungLabel->font() );
        Verlauf_BeschreibungLabel_font.setFamily(FontFamily);
	Verlauf_BeschreibungLabel->setFont( Verlauf_BeschreibungLabel_font ); 
	QFont Ergebnisse_TitelLabel_font(  Ergebnisse_TitelLabel->font() );
        Ergebnisse_TitelLabel_font.setFamily(FontFamily);
	Ergebnisse_TitelLabel_font.setBold( TRUE );
	Ergebnisse_TitelLabel->setFont( Ergebnisse_TitelLabel_font ); 
	QFont Ergebnisse_VarianzLabel_font(  Ergebnisse_VarianzLabel->font() );
	Ergebnisse_VarianzLabel_font.setBold( TRUE );
        Ergebnisse_VarianzLabel_font.setFamily(FontFamily);
	Ergebnisse_VarianzLabel->setFont( Ergebnisse_VarianzLabel_font ); 
	QFont Ergebnisse_Label2_font(  Ergebnisse_Label2->font() );
	Ergebnisse_Label2_font.setPointSize(PointSize);
	Ergebnisse_Label2_font.setBold( TRUE );
        Ergebnisse_Label2_font.setFamily(FontFamily);
	Ergebnisse_Label2->setFont( Ergebnisse_Label2_font ); 
	QFont Ergebnisse_BeschreibungLabel_font(  Ergebnisse_BeschreibungLabel->font() );
        Ergebnisse_BeschreibungLabel_font.setFamily(FontFamily);
	Ergebnisse_BeschreibungLabel->setFont( Ergebnisse_BeschreibungLabel_font ); 
	QFont Animation_TitelLabel_font(  Animation_TitelLabel->font() );
        Animation_TitelLabel_font.setFamily(FontFamily);
	Animation_TitelLabel_font.setBold( TRUE );
	Animation_TitelLabel->setFont( Animation_TitelLabel_font ); 
	QFont Animation_BeschreibungLabel_font(  Animation_BeschreibungLabel->font() );
        Animation_BeschreibungLabel_font.setFamily(FontFamily);
	Animation_BeschreibungLabel->setFont( Animation_BeschreibungLabel_font );  
  */
}

void PositronImp::init()
{
    overflow=0;
   leseDateiNamen();

   initEinfuehrung();
   initAuswahl();
   initStartwerte();
  
    
    Notebook->setTabEnabled ( VerlaufTab, false );
    Notebook->setTabEnabled (ErgebnisseTab,false);
    Notebook->setTabEnabled(AnimationTab,false);
}
void PositronImp::Animation_Slider_slot(int nr)
{
   aktualisiereAnimation(nr); 
}

/* 
 * public slot
 */
void PositronImp::Auswahl_EigeneSpektren_slot(int nr)
{
    Notebook->setTabEnabled ( VerlaufTab, false );
    Notebook->setTabEnabled (ErgebnisseTab,false);
    Notebook->setTabEnabled(AnimationTab,false);
    int i;

    leseSpektrum(AnzahldervordefiniertenSpektren+nr);
    Spektrumgewaehlt=AnzahldervordefiniertenSpektren+nr;
    
    aktualisiereAuswahlGraphik(AnzahldervordefiniertenSpektren+nr);
   Startwerte_AnzahlderModelltermeComboBox->setCurrentItem(0);
   Startwerte_AnzahlderModellterme_slot(0);

    for (i=0;i<AnzahldervordefiniertenSpektren;i++)
    Auswahl_VordefinierteSpektrenListbox->setSelected(i,false);

    Startwerte_VordefinierteStartwerteCombobox->clear();
   for (i=0;i<AnzahlderVoreinstellungen[Spektrumgewaehlt];i++) Startwerte_VordefinierteStartwerteCombobox->insertItem(voreinstellungennamen[i]); 

   if (AnzahlderVoreinstellungen[Spektrumgewaehlt]!=0) {
     Startwerte_VordefinierteStartwerteCombobox->setCurrentItem(0);
     Startwerte_VordefinierteStartwerte_slot(0);
   }  
}
/* 
 * public slot
 */
void PositronImp::Auswahl_VordefinierteSpektren_slot(int nr)
{
  int i;
  //Wenn neues Spektrum gewaehlt, die Ergbnisstabs verstecken
  Notebook->setTabEnabled ( VerlaufTab, false );
  Notebook->setTabEnabled (ErgebnisseTab,false);
  Notebook->setTabEnabled(AnimationTab,false);
  
  
  leseSpektrum(nr); 
  Spektrumgewaehlt=nr;

  for (i=0;i<AnzahlderBenutzerSpektren;i++)
    Auswahl_EigeneSpektrenListbox->setSelected(i,false);

  aktualisiereAuswahlGraphik(nr);

   Startwerte_AnzahlderModelltermeComboBox->setCurrentItem(0);
   Startwerte_AnzahlderModellterme_slot(0);

   Startwerte_VordefinierteStartwerteCombobox->clear();
   for (i=0;i<AnzahlderVoreinstellungen[Spektrumgewaehlt];i++) Startwerte_VordefinierteStartwerteCombobox->insertItem(voreinstellungennamen[i]); 

   if (AnzahlderVoreinstellungen[Spektrumgewaehlt]!=0) {
     Startwerte_VordefinierteStartwerteCombobox->setCurrentItem(0);
     Startwerte_VordefinierteStartwerte_slot(0);
   }
  
}
/* 
 * public slot
 */
void PositronImp::aktualisiereAuswahlGraphik(int nr)
{
  int i;
  AuswahlGraphic=new AIPlot(AuswahlTab,"graphic",ndata,2);
  AuswahlGraphic->setBackgroundColor(240,240,240);
  AuswahlGraphic->setGridColor(0,0,0);
  AuswahlGraphic->setCommonColor(0,0,0);
  AuswahlGraphic->setAutoScale(FALSE);
  AuswahlGraphic->setXName("Kanalnummer des Vielkanalanalysators");
  AuswahlGraphic->setYName("log(Zaehlrate)");
  AuswahlGraphic->setGeometry(235,145,430,260);
  AuswahlGraphic->setPlotColor(0,255,0,0);
  AuswahlGraphic->setDoubleBuffer(TRUE);
  AuswahlGraphic->setkomma(true);
 
  AnimationGraphic=new AIPlot(AnimationTab,"graphic",ndata,2);
  AnimationGraphic->setBackgroundColor(240,240,240);
  AnimationGraphic->setGridColor(0,0,0);
  AnimationGraphic->setCommonColor(0,0,0);
  AnimationGraphic->setAutoScale(FALSE);
  AnimationGraphic->setXName("Kanalnummer des Vielkanalanalysators");
  AnimationGraphic->setYName("log(Zaehlrate)");
  AnimationGraphic->setGeometry(5,80,340,300);
  AnimationGraphic->setPlotColor(0,255,0,0);
  AnimationGraphic->show();
  AnimationGraphic->setDoubleBuffer(TRUE);
  AnimationGraphic->setkomma(true);
    
  AnimationGraphic2=new AIPlot(AnimationTab,"graphic",ndata,2);
  AnimationGraphic2->setBackgroundColor(240,240,240);
  AnimationGraphic2->setGridColor(0,0,0);
  AnimationGraphic2->setCommonColor(0,0,0);
  AnimationGraphic2->setAutoScale(FALSE);
  AnimationGraphic2->setXName("Kanalnummer des Vielkanalanalysators");
  AnimationGraphic2->setYName("Residuum");
  AnimationGraphic2->setGeometry(345,80,335,300);
  AnimationGraphic2->setPlotColor(0,255,0,0);
  AnimationGraphic2->show();
  AnimationGraphic2->setDoubleBuffer(TRUE);
  AnimationGraphic2->setkomma(false);

  Auswahl_SpektrumTitelLabel->setText(spektrumnamen[nr]);

  yplot[0]=yplot[1]; //Workaround, da nrutil verwendet

  double ymax=0;
  double ymin=2000;
  for (i=1;i<=ndata;i++) {
     if (yplot[i]<ymin) ymin=yplot[i];
     if (yplot[i]>ymax) ymax=yplot[i];
   }

   AuswahlGraphic->setRanges(0,ndata,ymin-1,ymax+0.3);
   AuswahlGraphic->setData(0,ndata,xmess,yplot);
   AuswahlGraphic->show();
   AuswahlGraphic->repaint(FALSE);  
    

}

void PositronImp::Auswahl_Weiter_slot()
{
    Notebook->setCurrentPage(2);
}
/* 
 * public slot
 */
void PositronImp::Auwahl_SpektrumSimulieren_slot()
{
    SpektrumSimulierenImpl *simul;
    simul=new SpektrumSimulierenImpl(this,"",true);
    simul->show();
    init();
 
}
/* 
 * public slot
 */
void PositronImp::Beenden_slot()
{
   close();
}
/* 
 * public slot
 */
void PositronImp::Hilfe_slot()
{
    QString home;

    QDir d("");
    home=d.currentDirPath();
    home += "/doc/html/index.html";
    
    HelpWindow *help = new HelpWindow(home, "", 0, "Hilfe zu Positron");
    help->show();
}
/* 
 * public slot
 */
void PositronImp::Startwerte_AnzahlderIterationen_slot(int nr)
{
    int i;
    tmax=nr;
    VerlaufTable->setNumRows(tmax); 
    VerlaufTable->setNumCols(jmax*2+3);
    for (i=0;i<jmax*2+3;i++) VerlaufTable->setColumnWidth(i,60);    
    QHeader *horizheader;
    horizheader=VerlaufTable->horizontalHeader();
    QString headerlabel;
    horizheader->setLabel(0,"chisq");
    horizheader->setLabel(2*jmax+1,"Untergrund");
    horizheader->setLabel(2*jmax+2,"Marquardt");
    for (i=0;i<jmax;i++) {
      headerlabel="";
      headerlabel.sprintf("%d. Intensitt",i+1);
      horizheader->setLabel(i+1,headerlabel);
    }
    for (i=0;i<jmax;i++) {
      headerlabel="";
      headerlabel.sprintf("%d. Lebensd.",i+2);
      horizheader->setLabel(jmax+i+1,headerlabel);
    }

}
/* 
 * public slot
 */
void PositronImp::Startwerte_AnzahlderModellterme_slot(int nr)
{
    Voreinstellunggewaehlt=-1;
    QString headerlabel;
    int i;
    jmax=nr+1;
    Startwerte_Table->setNumRows(1);
    QHeader* vertheader=Startwerte_Table->verticalHeader();
    QHeader* horizheader=Startwerte_Table->horizontalHeader();
    Startwerte_Table->setNumRows(2*jmax+1);
    Startwerte_Table->setNumCols(1);
    Startwerte_Table->setColumnStretchable(0,true);
    Startwerte_Table->setText(0,0,"");
    Startwerte_Table->setText(1,0,"");
    vertheader->setMinimumWidth(115);
    horizheader->hide();
    vertheader->setLabel(2*jmax,"Untergrund");
    for (i=0;i<jmax;i++) {
      headerlabel="";
      headerlabel.sprintf("%d. Intensitt*10^-6",i+1);
      vertheader->setLabel(i,headerlabel);
    }
    for (i=0;i<jmax;i++) {
      headerlabel="";
      headerlabel.sprintf("%d. Lebensd.(ns)",i+1);
      vertheader->setLabel(jmax+i,headerlabel);
    }
    Startwerte_AnzahlderIterationen_slot(tmax);

    horizheader=Ergebnisse_Table->horizontalHeader();
    vertheader=Ergebnisse_Table->verticalHeader();
    Ergebnisse_Table->setLeftMargin(80);
    Ergebnisse_Table->setNumCols(2);
    Ergebnisse_Table->setNumRows(jmax*2+1);
    Ergebnisse_Table->setColumnStretchable(1,true);
    for (i=0;i<2*jmax+2;i++) {
      //Ergebnisse_KovarianzTable->setColumnWidth(i,65);
       //Ergebnisse_KovarianzTable->setColumnStretchable(i,true);
    }
    vertheader->setMinimumWidth(80);
    vertheader->setLabel(2*jmax,"Untergrund");
    horizheader->setLabel(0,"opt. Werte");
    horizheader->setLabel(1,"Standardabw.");  
    for (i=0;i<jmax;i++) {
      headerlabel="";
      headerlabel.sprintf("%d. Intensitt",i+1);
      vertheader->setLabel(i,headerlabel);
    }
    for (i=0;i<jmax;i++) {
      headerlabel="";
      headerlabel.sprintf("%d. Lebensd.(ns)",i+1);
      vertheader->setLabel(jmax+i,headerlabel);
    }


}
/* 
 * public slot
 */
void PositronImp::Startwerte_AuswertungStarten_slot()
{
   QString tabellentext;
   double del,delmax;
   int flag=0;
   int eingaberichtig=1;
   QString msg="";
   QString text="";
   overflow=0;
       
   gen=pow(10,(double) Startwerte_GenauigkeitSpinBox->value()); 
  
    alamda=-1.0;
    int t=1,flag2;
    tatit=0;
    int ma=jmax*2+2;
    int i,j;
    double *chisqplot;
    double *xplot;

    for (i=0;i<VerlaufTable->numRows();i++)
      for (j=0;j<VerlaufTable->numCols();j++) VerlaufTable->setText(i,j,"");

    chisqverlauf=dvector(1,tmax);
    beta=dvector(1,ma);
    a=dvector(1,ma);
    astart=dvector(1,ma);
    aalt=dvector(1,ma);
    covar=dmatrix(1,ma,1,ma);
    alpha=dmatrix(1,ma,1,ma); 
    for (i=1;i<=jmax;i++) {
      if (!Startwerte_Table->text(i-1,0).toDouble()) eingaberichtig=0;
      if (Startwerte_Table->text(i-1,0).toDouble()<0) eingaberichtig=-1;
      a[i]=Startwerte_Table->text(i-1,0).toDouble()*pow(10,6);
      astart[i]=a[i];
    }
    
    for (i=jmax+1;i<=2*jmax;i++) {
      if (!Startwerte_Table->text(i-1,0).toDouble()) eingaberichtig=0;
      if (Startwerte_Table->text(i-1,0).toDouble()<0) eingaberichtig=-1;
      a[i]=Startwerte_Table->text(i-1,0).toDouble();
      astart[i]=a[i];
    }
    if (!Startwerte_Table->text(2*jmax,0).toDouble()) eingaberichtig=0;
    if (Startwerte_Table->text(2*jmax,0).toDouble()<0) eingaberichtig=-1;

    a[2*jmax+1]=Startwerte_Table->text(2*jmax,0).toDouble();
    astart[2*jmax+1]=a[2*jmax+1];
    a[ma]=sigma;
    astart[ma]=sigma;
    if (eingaberichtig==1) {
      Notebook->setTabEnabled ( VerlaufTab, true );
      Notebook->setTabEnabled (ErgebnisseTab,true);
      Notebook->setTabEnabled(AnimationTab,true);
      Notebook->setCurrentPage(3);
      setCursor(QCursor(WaitCursor));
      for (j=1;j<=ma;j++) aalt[j]=a[j]; 
      do {
        if (overflow==1) {
		  overflow=0;
          flag=-2;
          break;
        }
           if (mrqmin(xmess,ymess,sig,ndata,ma,a,alpha,covar,beta,&chisq,&ochisq,&alamda)==1) {
              tabellentext=VerlaufTable->text(t-1,2*jmax+3);
              tabellentext+="*";
	      VerlaufTable->setText(t-1,2*jmax+2,tabellentext);
          }
          else {
             chisqverlauf[t]=chisq;
             tabellentext.sprintf("%.3lf",chisq);
             VerlaufTable->setText(t-1,0,tabellentext);
              for (j=1;j<=jmax;j++) {
                  tabellentext.sprintf("%.0lf",a[j]);
	          VerlaufTable->setText(t-1,j,tabellentext);
	      }
              for (j=jmax+1;j<=2*jmax;j++) {
                 tabellentext.sprintf("%2.3lf",a[j]);
                 VerlaufTable->setText(t-1,j,tabellentext);
	      }
               tabellentext.sprintf("%.2lf",a[2*jmax+1]);
              VerlaufTable->setText(t-1,2*jmax+1,tabellentext);
               
              delmax=0.0;
              for (j=1;j<=ma;j++) {
   	         del=fabs((a[j]-aalt[j])/a[j]);
     	         if (del>delmax) delmax=del;
              } 
              tatit=t;
                Animation_Slider->setMinValue(0);
                 Animation_Slider->setTickInterval(1);
                 Animation_Slider->setMaxValue(tatit);
                 text.sprintf("%d",tatit);
                 Animation_StopLabel->setText(text);
              if (delmax<gen) {
                 for (i=0;i<ma+2;i++) VerlaufTable->adjustColumn(i);
                 flag=1;
                 Notebook->setCurrentPage(4);
                 alamda=0.0;
    	         mrqmin(xmess,ymess,sig,ndata,ma,a,alpha,covar,beta,&chisq,&ochisq,&alamda);
                 Animation_Slider->setValue(0);
                 Animation_Slider_slot(0);
                 chisqplot=dvector(0,tatit-1);
                 xplot=dvector(0,tatit-1);
                 for(i=1;i<=tatit;i++) xplot[i-1]=i;
                 for (i=0;i<tatit;i++) chisqplot[i]=chisqverlauf[i+1];
                 ChisqGraphic=new AIPlot(ErgebnisseTab,"graphic",tatit,1);
                 ChisqGraphic->setBackgroundColor(240,240,240);
                 ChisqGraphic->setGridColor(0,0,0);
                 ChisqGraphic->setCommonColor(0,0,0);
                 ChisqGraphic->setAutoScale(FALSE);
                 ChisqGraphic->setXName("Nummer der Iteration");
                 ChisqGraphic->setYName("chisq");
                 ChisqGraphic->setGeometry(305,90,350,200);
                 ChisqGraphic->setPlotColor(0,255,0,0);
                 ChisqGraphic->show();
                 ChisqGraphic->setkomma(false);
                 ChisqGraphic->setDoubleBuffer(TRUE);
                 double ymax=0;
                 for (i=0;i<tatit;i++) {
                    if (chisqplot[i]>ymax) ymax=chisqplot[i];
                 }
                 ChisqGraphic->setRanges(1,tatit,0,ymax);
                 ChisqGraphic->setData(0,tatit,xplot,chisqplot);
                 ChisqGraphic->repaint(FALSE);  
                 VerlaufTable->setNumRows(tatit);
                 t=tmax+1;
                 for(i=1;i<=jmax;i++) {
	            tabellentext.sprintf("%.0lf",a[i]);
	            Ergebnisse_Table->setText(i-1,0,tabellentext);
	            tabellentext.sprintf("%.0lf",sqrt(covar[i][i]));
                    Ergebnisse_Table->setText(i-1,1,tabellentext);
                 } 
                 for(i=jmax+1;i<=jmax*2;i++) {
                   tabellentext.sprintf("%.3lf",a[i]);
                   Ergebnisse_Table->setText(i-1,0,tabellentext);
                   tabellentext.sprintf("%.3lf",sqrt(covar[i][i]));
                   Ergebnisse_Table->setText(i-1,1,tabellentext);
                 }
                 tabellentext.sprintf("%.2lf",a[2*jmax+1]);
                 Ergebnisse_Table->setText(2*jmax,0,tabellentext);
                 tabellentext.sprintf("%.2lf",sqrt(covar[2*jmax+1][2*jmax+1]));
                 Ergebnisse_Table->setText(2*jmax,1,tabellentext);
                var=chisq/(ndata-ma);
                sdvar=sqrt(2./(ndata-ma));
                double prozent=sdvar/100;
                bool abfrage=(var>=1-sdvar+prozent)&&(var<=1+sdvar-prozent);
                //printf("%f\n",var);
                //printf("%f\n",sdvar);
                if (abfrage==true) Ergebnisse_VarianzLabel->setText("Gutes Modell");
                else if (((var>=1-sdvar-prozent)&&(var<=1-sdvar+prozent))||((var>=1+sdvar-prozent)&&(var<=1+sdvar+prozent)))
                   Ergebnisse_VarianzLabel->setText("Keine Aussage ber Modellqualitt");
                else {
                  flag2=0;
                  for (i=1;i<=2*jmax+1;i++) {
                    if (sqrt(covar[i][i]) > a[i]) flag2=1;
		  }
		  if (flag2==1)
		    Ergebnisse_VarianzLabel->setText("Schlechtes Modell (beachten sie die Standardabweichungen)");
                  else if (flag2==0&&jmax==VordefinierteAnzahlderZerfallsterme[Spektrumgewaehlt])
                    Ergebnisse_VarianzLabel->setText("Schlechtes Modell (vermutlich Nebenminimum von chisq)");
                  else 
                    Ergebnisse_VarianzLabel->setText("Schlechtes Modell (vermutlich zu wenige Modellterme)");
                }

                if (Voreinstellunggewaehlt>-1) Ergebnisse_BeschreibungTextView->setText(voreinstellungenergebnisstext[Voreinstellunggewaehlt]); else Ergebnisse_BeschreibungTextView->setText(""); 
                if (AnzahlderVoreinstellungen[Spektrumgewaehlt]==0&&abfrage==true&&jmax==VordefinierteAnzahlderZerfallsterme[Spektrumgewaehlt]) Ergebnisse_BeschreibungTextView->setText(voreinstellungenergebnisstext[0]); 
                double annen;
                for (i=1;i<=ma-1;i++) {
	           for (j=i+1;j<=ma;j++) {
	             annen=sqrt(covar[i][i]*covar[j][j]);
	             covar[i][j]=covar[i][j]/annen;
	             covar[j][i]=covar[i][j];
                  }
                }
         
             }
             else {
                 for (j=1;j<=ma;j++) aalt[j]=a[j];
                 t++;
              }
      }
   } while (t<=tmax);
   setCursor(QCursor(ArrowCursor));
   if (flag==0) {
      msg="Gewnschte Genauigkeit wurde nicht erreicht\n Erhhen Sie die Anzahl der maximalen Iterationen oder geben Sie andere Startwerte an.";
      msg+=" \n Die Ergebnisse stehen deswegen nicht zur Verfgung";
       Notebook->setTabEnabled (ErgebnisseTab,false); 
       for (i=0;i<ma;i++) VerlaufTable->adjustColumn(i);
      QMessageBox::information(0,"Fehler",msg,QMessageBox::Ok);
       Animation_Slider->setValue(0);
       Animation_Slider_slot(0);

   } else if (flag==-1) {
      msg="Auswertung wurde unterbrochen";
      msg+=" \n Die Ergebnisse stehen deswegen nicht zur Verfgung";
       Notebook->setTabEnabled (ErgebnisseTab,false); 
       for (i=0;i<ma;i++) VerlaufTable->adjustColumn(i);
      QMessageBox::information(0,"Fehler",msg,QMessageBox::Ok);
       Animation_Slider->setValue(0);
       Animation_Slider_slot(0);
   } else if (flag==-2) {
        msg="Es ist ein Exponenten-Overflow aufgetreten";
      msg+=" \n Bitte whlen Sie andere Startwerte";
       overflow=0;
       Notebook->setTabEnabled (ErgebnisseTab,false);
       Notebook->setTabEnabled (AnimationTab,false); 
       Notebook->setTabEnabled (VerlaufTab,false);
       for (i=0;i<ma;i++) VerlaufTable->adjustColumn(i);
      QMessageBox::information(0,"Fehler",msg,QMessageBox::Ok);
      Notebook->setCurrentPage(2);
   }
  
   } else {
     if (eingaberichtig==0) msg="Fehlerhafte Eingabe der Startwerte\n Konnte Eingabe nicht zu Zahl konvertieren";
     else if (eingaberichtig==-1) msg="Fehlerhafte Eingabe der Startwerte\n Startwerte drfen nicht negativ sein";
      QMessageBox::information(0,"Fehler",msg,QMessageBox::Ok);
   }
}
/* 
 * public slot
 */
void PositronImp::Startwerte_Genauigkeit_slot(int nr)
{
}
/* 
 * public slot
 */
void PositronImp::Startwerte_VordefinierteStartwerte_slot(int nummer)
{
    int i;
    QString dummy;
    Startwerte_AnzahlderModelltermeComboBox->setCurrentItem(voreinstellungenjmax[nummer]-1);
    Startwerte_AnzahlderModellterme_slot(voreinstellungenjmax[nummer]-1);
    for (i=0;i<voreinstellungenjmax[nummer]*2+1;i++) {
      dummy.sprintf("%.3lf",voreinstellungenstartwerte[nummer][i]);
      Startwerte_Table->setText(i,0,dummy);
    }    
   Voreinstellunggewaehlt=nummer;
}
/* 
* public slot
 */
void PositronImp::Animation_Play_slot()
{
    for (int i=0;i<=tatit;i++) {
      Animation_Slider->setValue(i);
#ifdef _WS_X11_
      usleep(300000);
#endif
#ifdef _WS_WIN_
	  Sleep(300);
#endif
    }
}

void PositronImp::Animation_Stop_slot()
{
}

void PositronImp::Einfuehrung_Weiter_slot()
{
  Notebook->setCurrentPage(1);
}

void PositronImp::initStartwerte(){
   QString dummy;
 
   Startwerte_AnzahlderModelltermeComboBox->clear();
   int i;
   for (i=1;i<=5;i++) {
     dummy.sprintf("%d",i);
     Startwerte_AnzahlderModelltermeComboBox->insertItem(dummy);
   }

    Startwerte_Table->setTopMargin(0);
    Startwerte_Table->setLeftMargin(115);
    Startwerte_Table->setColumnMovingEnabled(0);
    Startwerte_Table->setRowMovingEnabled(0);
    Startwerte_Table->setColumnStretchable (0,true);

    //Startwerte_AnzahlderModelltermeComboBox->setCurrentItem(0);
    //eStartwerte_AnzahlderModellterme_slot(0);

    Startwerte_AnzahlderIterationenSpinBox->setMinValue(1);
    Startwerte_AnzahlderIterationenSpinBox->setMaxValue(1000);
    Startwerte_AnzahlderIterationenSpinBox->setValue(100);

    Startwerte_GenauigkeitSpinBox->setMinValue(-12);
    Startwerte_GenauigkeitSpinBox->setMaxValue(-1);
    Startwerte_GenauigkeitSpinBox->setValue(-6);

}

void PositronImp::initEinfuehrung() {
     Einfuehrung_TextBrowser->mimeSourceFactory()->setFilePath("doc/html");
     Einfuehrung_TextBrowser->setSource("intro2.html");
}

void PositronImp::initAuswahl(){
   int i;
   //Auswahl_VordefinierteSpektrenListbox fllen
   for(i=0;i<AnzahldervordefiniertenSpektren;i++) leseSpektrumName(i);
   Auswahl_VordefinierteSpektrenListbox->clear();
   for (i=0;i<AnzahldervordefiniertenSpektren;i++) Auswahl_VordefinierteSpektrenListbox->insertItem(spektrumnamen[i]);
   Auswahl_VordefinierteSpektrenListbox->setSelected(2,true);
   Auswahl_VordefinierteSpektren_slot(2); //Sonst wird Event anscheinend nicht ausgelst.

   //Auswahl_EigeneSpektrenListbox fllen
   for(i=0;i<AnzahlderBenutzerSpektren;i++) leseSpektrumName(AnzahldervordefiniertenSpektren+i);
   Auswahl_EigeneSpektrenListbox->clear();
   for (i=0;i<AnzahlderBenutzerSpektren;i++) 
     Auswahl_EigeneSpektrenListbox->insertItem(spektrumnamen[AnzahldervordefiniertenSpektren+i]);
}

int PositronImp::leseDateiNamen(){
   QString datenpfad;
   QString dateiname;
   datenpfad="daten";
   int i;
   QDir d(datenpfad);

   d.setNameFilter("*.dat");
   d.setSorting(QDir::Name||QDir::Reversed);
   AnzahldervordefiniertenSpektren=d.count();
   if (AnzahldervordefiniertenSpektren>SPEKTRUMMAX) AnzahldervordefiniertenSpektren=SPEKTRUMMAX;
   for (i=0;i<AnzahldervordefiniertenSpektren;i++) {
    dateiname=d.path();
    dateiname+="/";
    dateiname+=d[i];
    dateinamen[i]=dateiname;
   }

   d.setNameFilter("*.usr");
   AnzahlderBenutzerSpektren=d.count();
   if (AnzahlderBenutzerSpektren>SPEKTRUMMAX) AnzahlderBenutzerSpektren=SPEKTRUMMAX;
   for (i=0;i<AnzahlderBenutzerSpektren;i++) {
    dateiname=d.path();
    dateiname+="/";
    dateiname+=d[i];
    dateinamen[AnzahldervordefiniertenSpektren+i]=dateiname;
   }
   return 1;
}
int PositronImp::leseSpektrumName(int nummer){
    QFile datei;
    QString msg;
      if (!dateinamen[nummer].isEmpty()){
        datei.setName(dateinamen[nummer]);
        if (!datei.open(IO_ReadOnly)) { 
          msg="";
          msg+="Datei ";
          msg+=dateinamen[nummer];
          msg+=" kann nicht geffnet werden";
	  QMessageBox::information(0,"Fehler beim ffnen der Datei",msg,QMessageBox::Ok);
          return 0;  
        } else { 
          QTextStream instream(&datei);
          spektrumnamen[nummer]=instream.readLine();
          datei.close();
          return 1;
        }
    }
}
void PositronImp::aktualisiereAnimation(int frame) {
  int i;
  double *yplot2;
  double *residuumplot;
  double *residuumhelp;
  double *atest;
  double *dydatest;
  int ma=2*jmax+2;
  dydatest=dvector(1,ma);
  atest=dvector(1,ma);
  yplot2=dvector(0,ndata-1);
  residuumhelp=dvector(0,ndata-1);
  residuumplot=dvector(0,ndata-1);
  //Auswahl_SpektrumTitelLabel->setText(spektrumnamen[nr]);
  yplot[0]=yplot[1]; //Workaround, da nrutil verwendet
  double ymax=0;
  double ymin=2000;
  for (i=1;i<=ndata;i++) {
     if (yplot[i]<ymin) ymin=yplot[i];
     if (yplot[i]>ymax) ymax=yplot[i];
   }
   AnimationGraphic->setRanges(0,ndata,ymin-1,ymax+0.3);
   AnimationGraphic->setData(0,ndata,xmess,yplot);
   AnimationGraphic->setPlotColor(1,0,0,255);
     for (i=1;i<ma;i++)  atest[i]=VerlaufTable->text(frame-1,i).toDouble();
     atest[ma]=sigma;
     for (i=0;i<ndata;i++) {
        if (frame!=0) funcs(xmess[i+1],atest,&yplot2[i],dydatest,ma);
        else funcs(xmess[i+1],astart,&yplot2[i],dydatest,ma);
        yplot2[i]=log(yplot2[i]);
     }
     AnimationGraphic->setData(1,ndata,xmess,yplot2);
   AnimationGraphic->repaint(FALSE);
 
    for (i=1;i<=ndata;i++) residuumplot[i-1]=ymess[i]-exp(yplot2[i-1]);  
       residuummax=0;
    residuummin=pow(10,100);
    for (i=0;i<=ndata-1;i++) {
            if (residuumplot[i]<residuummin) residuummin=residuumplot[i];
            if (residuumplot[i]>residuummax) residuummax=residuumplot[i];
    }
   AnimationGraphic2->setRanges(0,ndata,residuummin,residuummax);
   AnimationGraphic2->setData(0,ndata,xmess,residuumplot);
   AnimationGraphic2->repaint(FALSE);
   overflow=0;
}

void PositronImp::Ergebnisse_Kovarianz_slot() {
  KovarianzFormImpl *kovar;
    kovar=new KovarianzFormImpl();
    kovar->fuelle(covar,2*jmax+1);
    kovar->show();
}
void PositronImp::Ergebnisse_Weiter_slot()
{
  Notebook->setCurrentPage(5);
}
int PositronImp::leseSpektrum(int nummer){
    QFile datei;
    QString msg,dummystring;
    int i,j,dummy;
    if (!dateinamen[nummer].isEmpty()){
      datei.setName(dateinamen[nummer]);
      if (!datei.open(IO_ReadOnly)) {
        msg="";
        msg+="Datei ";
        msg+=dateinamen[nummer];
        msg+=" kann nicht geffnet werden";
	QMessageBox::information(0,"Fehler beim ffnen der Datei",msg,QMessageBox::Ok);
        return 0;  
      } else {
          QTextStream instream (&datei);
          instream.readLine();
          Auswahl_BeschreibungTextView->setText(instream.readLine());
          instream >>delta;
          instream >>t0;
          instream >>sigma;
          instream >> ndata;

          xmess=dvector(1,ndata);
          ymess=dvector(1,ndata);
          yplot=dvector(1,ndata);
          sig=dvector(1,ndata);
          for (i=1;i<=ndata;i++) {
             xmess[i]=i;
            instream>>ymess[i];
             sig[i]=sqrt(ymess[i]);
             yplot[i]=log(ymess[i]);
	    }
          instream >>AnzahlderVoreinstellungen[nummer];
          if (AnzahlderVoreinstellungen[nummer]>0) {
            VordefinierteAnzahlderZerfallsterme[nummer]=-1;
            for (i=0;i<AnzahlderVoreinstellungen[nummer];i++) {
              dummystring=instream.readLine();
              voreinstellungennamen[i]=instream.readLine();
	      voreinstellungenergebnisstext[i]=instream.readLine();
              instream>>voreinstellungenjmax[i];
              for (j=0;j<voreinstellungenjmax[i];j++) {
                 instream>>voreinstellungenstartwerte[i][j];
                voreinstellungenstartwerte[i][j]*=pow(10,-6);
              }
              for (j=voreinstellungenjmax[i];j<voreinstellungenjmax[i]*2+1;j++) {
              instream>>voreinstellungenstartwerte[i][j];
              }
              voreinstellungenstartwerte[i][voreinstellungenjmax[i]*2+1]=sigma;
            }
          } else {
            Voreinstellunggewaehlt=-1;
            instream>>VordefinierteAnzahlderZerfallsterme[nummer];
            dummystring=instream.readLine();
            
            voreinstellungenergebnisstext[0]=instream.readLine();
          }
      } 
     
          datei.close();
          return 1;
        }    
    }

bool PositronImp::event( QEvent* ev )
{
    bool ret = QWidget::event( ev ); 
      if ( ev->type() == QEvent::ApplicationFontChange ) {
      initFonts();
       }
    return ret;
}
